from __future__ import absolute_import

import contextlib
import logging
import os
import subprocess
import sys
import tempfile

from ansible_toolbox.base import BaseApp

LOG = logging.getLogger(__name__)


@contextlib.contextmanager
def temporary_file(*args, **kwargs):
    '''Return a temporary filename.

    Generate a temporary filename, return it to the caller, and clean it up
    when done.  Unlike tempfile.NamedTemporaryFile and friends, you only get
    back a filename rather than a file object.
    '''
    name = tempfile.mktemp(*args, **kwargs)
    yield name
    os.unlink(name)


class EvalApp (BaseApp):
    '''Evaluate a Jinja2 template and display the result.'''

    def build_argument_parser(self):
        p = super(EvalApp, self).build_argument_parser()
        p.add_argument('expr')
        return p

    def main(self, args):
        template = self.get_template('eval.yml')

        with tempfile.NamedTemporaryFile(dir='.', suffix='.yml') as tmplfd, \
                temporary_file(dir='.') as output:
            playbook = template.render(
                    expr=args.expr,
                    hosts=args.hosts,
                    gather=args.gather,
                    output=output,
            )

            LOG.debug('playbook: %s', playbook)
            tmplfd.write(playbook.encode('utf-8'))
            tmplfd.flush()

            cmd = ['ansible-playbook', tmplfd.name]
            cmd.extend(self.build_command_line(args))
            LOG.debug('running command: %s', cmd)
            subprocess.check_output(cmd)
            with open(output, 'r') as fd:
                sys.stdout.write(fd.read())
                sys.stdout.write('\n')


app = EvalApp()


def main():
    app.run()



import argparse
import jinja2
import jinja2.loaders
import logging
import os
import subprocess
import sys


class BaseApp (object):
    def __init__(self):
        self._env = jinja2.Environment(
            loader=jinja2.loaders.PackageLoader('ansible_toolbox',
                                                'templates'))

    def get_template(self, template):
        return self._env.get_template(template)

    def build_argument_parser(self):
        p = argparse.ArgumentParser()
        p.add_argument('--verbose', '-v',
                       action='count',
                       default=0)
        p.add_argument('--debug',
                       action='store_true')

        p.add_argument('--gather', '-g',
                       action='store_true')
        p.add_argument('--no-gather', '-G',
                       action='store_false',
                       dest='gather')
        p.add_argument('--extra-vars', '-e',
                       action='append',
                       default=[])
        p.add_argument('--connection', '-c')
        p.add_argument('--check', '-C', action='store_true')

        g = p.add_argument_group('Inventory')
        g.add_argument('-i', '--inventory')
        g.add_argument('--hosts', '-H',
                       default='localhost')

        g = p.add_argument_group('Identity')
        g.add_argument('--sudo', '-s',
                       action='store_true',
                       dest='become')
        g.add_argument('--become', '-b',
                       action='store_true')
        g.add_argument('--user', '-u')
        g.add_argument('--ask-become-pass', '-K',
                       action='store_true')

        p.set_defaults(gather=True)
        return p

    def parse_args(self):
        p = self.build_argument_parser()
        return p.parse_args()

    def build_command_line(self, args):
        cmd = []

        ap_args = [('-e', x) for x in args.extra_vars]
        if args.inventory:
            cmd.extend(('-i', args.inventory))

        if args.check:
            cmd.append('--check')

        if args.become:
            cmd.append('--become')

        if args.ask_become_pass:
            cmd.append('--ask-become-pass')

        if args.user:
            cmd.extend(('-u', args.user))

        if args.connection:
            cmd.extend(('-c', args.connection))

        for i in range(args.verbose):
            cmd.append('-v')

        for arg in ap_args:
            cmd.extend(arg)

        return cmd

    def probe_ansible_version(self):
        '''Return information about the installed version of Ansible.

        We need to know the current Ansible version in order to avoid
        deprecation warnings or other issues in our generated playbooks.
        '''
        out = subprocess.check_output(['ansible-playbook', '--version'])
        version = out.decode('utf-8').splitlines()[0].split()[1]
        return tuple(int(x) for x in version.split('.'))

    def run(self):
        args = self.parse_args()
        res = 0
        os.environ["ANSIBLE_RETRY_FILES_ENABLED"] = "False"

        logging.basicConfig(
            level='DEBUG' if args.debug else 'INFO')

        try:
            self.main(args)
        except subprocess.CalledProcessError as err:
            res = err.returncode
            if err.output:
                sys.stdout.write(err.output.decode('utf-8'))
        except KeyboardInterrupt:
            res = 1

        sys.exit(res)



from __future__ import absolute_import

import logging
import subprocess
import tempfile

from ansible_toolbox.base import BaseApp

LOG = logging.getLogger(__name__)


class RoleApp (BaseApp):
    '''Execute a single Ansible role.'''

    def build_argument_parser(self):
        p = super(RoleApp, self).build_argument_parser()
        p.add_argument('-t', '--tags')
        p.add_argument('role')
        return p

    def build_command_line(self, args):
        cmd = super(RoleApp, self).build_command_line(args)

        if args.tags:
            cmd.extend(('-t', args.tags))

        return cmd

    def main(self, args):
        template = self.get_template('role.yml')

        with tempfile.NamedTemporaryFile(dir='.') as fd:
            playbook = template.render(
                role=args.role,
                hosts=args.hosts,
                gather=args.gather,
            )

            LOG.debug('playbook: %s', playbook)
            fd.write(playbook.encode('utf-8'))
            fd.flush()

            cmd = ['ansible-playbook', fd.name]
            cmd.extend(self.build_command_line(args))
            LOG.debug('running command: %s', cmd)
            subprocess.check_call(cmd)


app = RoleApp()


def main():
    app.run()



from setuptools import setup, find_packages
from ansible_toolbox import __version__ as version

with open('requirements.txt') as fd:
    setup(name='ansible-toolbox',
          author='Lars Kellogg-Stedman',
          author_email='lars@oddbit.com',
          url='https://github.com/larsks/ansible-toolbox',
          version=version,
          install_requires=fd.readlines(),
          packages=find_packages(),
          package_data={'ansible_toolbox': [
              'templates/*'
          ]},
          entry_points={'console_scripts': [
              'ansible-role = ansible_toolbox.cmd.role:main',
              'ansible-task = ansible_toolbox.cmd.task:main',
              'ansible-eval = ansible_toolbox.cmd.eval:main',
          ]},
          )
